package cn.com.qmth.stmms.admin.thread;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.stmms.biz.exam.model.CheckStudent;
import cn.com.qmth.stmms.biz.exam.model.Exam;
import cn.com.qmth.stmms.biz.exam.model.ExamQuestion;
import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.exam.service.CheckStudentService;
import cn.com.qmth.stmms.biz.exam.service.ExamQuestionService;
import cn.com.qmth.stmms.biz.exam.service.ExamService;
import cn.com.qmth.stmms.biz.exam.service.ExamStudentService;
import cn.com.qmth.stmms.biz.lock.LockService;
import cn.com.qmth.stmms.biz.mark.service.MarkService;
import cn.com.qmth.stmms.biz.report.service.ReportService;
import cn.com.qmth.stmms.biz.utils.ScoreCalculateUtil;
import cn.com.qmth.stmms.biz.utils.ScoreInfo;
import cn.com.qmth.stmms.common.enums.CheckType;
import cn.com.qmth.stmms.common.enums.LockType;

public class ScoreCalculateThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(ScoreCalculateThread.class);

    private ExamStudentService studentService;

    private ExamQuestionService questionService;

    private MarkService markService;

    private int examId;

    private LockService lockService;

    private ReportService reportService;

    private Map<String, List<ExamQuestion>> objectiveMap;

    private Map<String, List<ExamQuestion>> subjectiveMap;

    private CheckStudentService checkStudentService;
    
    private ExamService examService;

    public ScoreCalculateThread(int examId, LockService lockService, ExamStudentService studentService,
            ExamQuestionService questionService, 
            MarkService markService, CheckStudentService checkStudentService,
            ReportService reportService,ExamService examService) {
        this.examId = examId;
        this.lockService = lockService;
        this.studentService = studentService;
        this.questionService = questionService;
        this.markService = markService;
        this.checkStudentService = checkStudentService;
        this.reportService = reportService;
        this.examService = examService;
        this.objectiveMap = new HashMap<String, List<ExamQuestion>>();
        this.subjectiveMap = new HashMap<String, List<ExamQuestion>>();
    }

    @Override
    public void run() {
        log.info("start calculate for examId=" + examId);
        try {
            int pageNumber = 1;
            int pageSize = 1000;
            List<ExamStudent> list = studentService.findByExamIdAndUploadAndAbsent(examId, true, false, pageNumber,
                    pageSize);
            while (list != null && list.size() > 0) {
                for (ExamStudent student : list) {
                    calculate(student);
                }

                pageNumber++;
                list = studentService.findByExamIdAndUploadAndAbsent(examId, true, false, pageNumber, pageSize);
            }

            // 统计识别缺考的试卷
            checkStudentService.deleteByExamId(examId);
            List<ExamStudent> students = studentService.findByExamIdAndUploadAndAbsent(examId, true, true, 1,
                    Integer.MAX_VALUE);
            for (ExamStudent examStudent : students) {
                CheckStudent student = new CheckStudent(examStudent.getId(), examId, examStudent.getSubjectCode(),
                        CheckType.SCAN_ABSENT);
                checkStudentService.save(student);
            }
            
            // 生成报表数据
            reportService.generateReportData(examId);
            //统分结束
            Exam exam = examService.findById(examId);
            exam.setNeedCalculate(false);
            examService.save(exam);
        } catch (Exception e) {
            log.error("calculate exception for examId=" + examId, e);
        } finally {
            lockService.unlock(LockType.EXAM, examId);
            log.info("finish calculate for examId=" + examId);
        }
    }

    private void calculate(ExamStudent student) {
        try {
            ScoreCalculateUtil util = ScoreCalculateUtil.instance(student);
            ScoreInfo info = util.calculate(findQuestionList(student.getSubjectCode(), student.getPaperType(), true),
                    null);

            student.setObjectiveScore(info.getObjectiveScore());
            student.setScoreList(info.getScoreList(), true);

            studentService.save(student);

            // 增加主观题总分统计
            markService.scoreCalculate(student.getExamId(), student.getSubjectCode(), student.getId());

            // 客观题0分主观题有分的考生
            if (student.getObjectiveScore() == 0 && student.getSubjectiveScore() != 0) {
                CheckStudent cs = new CheckStudent(student.getId(), examId, student.getSubjectCode(),
                        CheckType.EXCEPTION);
                checkStudentService.save(cs);
            }
        } catch (Exception e) {
            log.error("statistic error", e);
        }
    }

    private List<ExamQuestion> findQuestionList(String subjectCode, String paperType, boolean objective) {
        if (objective) {
            String key = subjectCode + "_" + StringUtils.trimToEmpty(paperType);
            List<ExamQuestion> list = objectiveMap.get(key);
            if (list == null) {
                list = questionService.findByExamAndSubjectAndObjectiveAndPaperType(examId, subjectCode, true,
                        paperType);
                objectiveMap.put(key, list);
            }
            return list;
        } else {
            List<ExamQuestion> list = subjectiveMap.get(subjectCode);
            if (list == null) {
                list = questionService.findByExamAndSubjectAndObjective(examId, subjectCode, false);
                subjectiveMap.put(subjectCode, list);
            }
            return list;
        }
    }
}
